@@ -86,6 +86,9 @@ static command_rec ssl_config_cmds[] = {
     AP_SRV_CMD(SessionCache, TAKE1,
                "SSL Session Cache storage "
                "(`none', `dbm:/path/to/file')")
+    AP_SRV_CMD(RandomSeed, TAKE23,
+               "SSL Pseudo Random Number Generator (PRNG) seeding source "
+               "(`startup|connect builtin|file:/path|exec:/path [bytes]')")
 
     /*
      * Per-server context configuration directives
@@ -93,7 +96,7 @@ static command_rec ssl_config_cmds[] = {
     AP_SRV_CMD(Engine, FLAG,
                "SSL switch for the protocol engine "
                "(`on', `off')")
-    AP_SRV_CMD(CipherSuite, TAKE1,
+    AP_ALL_CMD(CipherSuite, TAKE1,
                "Colon-delimited list of permitted SSL Ciphers "
                "(`XXX:...:XXX' - see manual)")
     AP_SRV_CMD(CertificateFile, TAKE1,
@@ -102,16 +105,25 @@ static command_rec ssl_config_cmds[] = {
     AP_SRV_CMD(CertificateKeyFile, TAKE1,
                "SSL Server Private Key file "
                "(`/path/to/file' - PEM encoded)")
+#ifdef SSL_EXPERIMENTAL
+    AP_ALL_CMD(CACertificatePath, TAKE1,
+               "SSL CA Certificate path "
+               "(`/path/to/dir' - contains PEM encoded files)")
+    AP_ALL_CMD(CACertificateFile, TAKE1,
+               "SSL CA Certificate file "
+               "(`/path/to/file' - PEM encoded)")
+#else
     AP_SRV_CMD(CACertificatePath, TAKE1,
                "SSL CA Certificate path "
                "(`/path/to/dir' - contains PEM encoded files)")
     AP_SRV_CMD(CACertificateFile, TAKE1,
                "SSL CA Certificate file "
                "(`/path/to/file' - PEM encoded)")
-    AP_SRV_CMD(VerifyClient, TAKE1,
+#endif
+    AP_ALL_CMD(VerifyClient, TAKE1,
                "SSL Client verify type "
                "(`none', `optional', `require', `optional_no_ca')")
-    AP_SRV_CMD(VerifyDepth, TAKE1,
+    AP_ALL_CMD(VerifyDepth, TAKE1,
                "SSL Client verify depth "
                "(`N' - number of intermediate certifcates)")
     AP_SRV_CMD(SessionCacheTimeout, TAKE1,
