@@ -84,6 +84,7 @@
 #include <pem.h>
 #include <crypto.h>
 #include <evp.h>
+#include <rand.h>
 
 /* Apache headers */
 #define CORE_PRIVATE
@@ -95,11 +96,13 @@
 #include "http_main.h"
 #include "http_core.h"
 #include "http_log.h"
+#include "scoreboard.h"
 #include "fnmatch.h"
 #undef CORE_PRIVATE
 
 /* mod_ssl headers */
 #include "ssl_expr.h"
+#include "ssl_util_ssl.h"
 
 /*
  * Provide reasonable default for some defines
@@ -156,6 +159,8 @@
 #define myCtxVarSet(mc,num,val)  mc->rCtx.pV##num = val
 #define myCtxVarGet(mc,num,type) (type)(mc->rCtx.pV##num)
 
+#define AP_ALL_CMD(name, args, desc) \
+        { "SSL"#name, ssl_cmd_SSL##name, NULL, RSRC_CONF|OR_AUTHCFG, args, desc },
 #define AP_SRV_CMD(name, args, desc) \
         { "SSL"#name, ssl_cmd_SSL##name, NULL, RSRC_CONF, args, desc },
 #define AP_DIR_CMD(name, type, args, desc) \
@@ -280,39 +285,16 @@
 #endif
 
 /*
- * Check for SSL library version number
- */
-#ifdef OPENSSL_VERSION_NUMBER
-#define SSL_LIBRARY_VERSION OPENSSL_VERSION_NUMBER
-#define SSL_LIBRARY_NAME    "OpenSSL"
-#else
-#ifdef SSLEAY_VERSION_NUMBER
-#define SSL_LIBRARY_VERSION SSLEAY_VERSION_NUMBER
-#define SSL_LIBRARY_NAME    "SSLeay"
-#else
-#define SSL_LIBRARY_VERSION 0x0000
-#define SSL_LIBRARY_NAME    "OtherSSL"
-#endif
-#endif
-
-/*
- * Check for SSLeay 0.8.0 and below
+ * Check for SSLeay 0.9.0 and below
  * and enabled Extended API (EAPI)
  */
-#if SSL_LIBRARY_VERSION < 0x0800
-#error "SSLeay versions below 0.8.0 are not supported"
+#if SSL_LIBRARY_VERSION < 0x0900
+#error "SSLeay versions below 0.9.0 are not supported"
 #endif
 #ifndef EAPI
 #error "mod_ssl requires Extended API (EAPI)"
 #endif
 
-/*
- * Correct a typo in old SSLeay
- */
-#if SSL_LIBRARY_VERSION < 0x900
-#define SSL3_TXT_ADH_DES_192_CBC_SHA SSL3_TXT_ADH_DES_196_CBC_SHA
-#endif
-
 /*
  * The own data structures
  */
@@ -405,6 +387,25 @@ typedef struct {
     ssl_expr *mpExpr;
 } SSLRequirement;
 
+/*
+ * Define the SSL random number generator seeding source
+ */
+typedef enum {
+    SSL_RSCTX_STARTUP = 1,
+    SSL_RSCTX_CONNECT = 2
+} SSLRandSeedCtx;
+typedef enum {
+    SSL_RSSRC_BUILTIN = 1,
+    SSL_RSSRC_FILE    = 2,
+    SSL_RSSRC_EXEC    = 3
+} SSLRandSeedSrc;
+typedef struct {
+    SSLRandSeedCtx  nCtx;
+    SSLRandSeedSrc  nSrc;
+    char           *cpPath;
+    int             nBytes;
+} SSLRandSeed;
+
 /*
  * Define the mod_ssl per-module configuration structure
  * (i.e. the global configuration for each httpd process)
@@ -412,6 +413,8 @@ typedef struct {
 
 typedef struct {
     pool         *pPool;
+    BOOL          bFixed;
+
     /*
      * global config data
      */
@@ -424,6 +427,7 @@ typedef struct {
     char         *szMutexFile;
     int           nMutexFD;
     int           nMutexSEMID;
+    array_header *aRandSeed;
 
     ssl_ds_table *tPublicCert;
     ssl_ds_table *tPrivateKey;
@@ -472,6 +476,13 @@ typedef struct {
     OptionType    nOptions;
     OptionType    nOptionsAdd;
     OptionType    nOptionsDel;
+    char         *szCipherSuite;
+    VerifyType    nVerifyClient;
+    int           nVerifyDepth;
+#ifdef SSL_EXPERIMENTAL
+    char         *szCACertificatePath;
+    char         *szCACertificateFile;
+#endif
 } SSLDirConfigRec;
 
 /*
@@ -483,20 +494,23 @@ extern module MODULE_VAR_EXPORT ssl_module;
 
 /*  configuration handling   */
 void         ssl_config_global_create(void);
+void         ssl_config_global_fix(void);
+BOOL         ssl_config_global_isfixed(void);
 void        *ssl_config_server_create(pool *, server_rec *);
 void        *ssl_config_server_merge(pool *, void *, void *);
 void        *ssl_config_perdir_create(pool *, char *);
 void        *ssl_config_perdir_merge(pool *, void *, void *);
 const char  *ssl_cmd_SSLMutex(cmd_parms *, char *, char *);
 const char  *ssl_cmd_SSLPassPhraseDialog(cmd_parms *, char *, char *);
+const char  *ssl_cmd_SSLRandomSeed(cmd_parms *, char *, char *, char *, char *);
 const char  *ssl_cmd_SSLEngine(cmd_parms *, char *, int flag);
-const char  *ssl_cmd_SSLCipherSuite(cmd_parms *, char *, char *);
+const char  *ssl_cmd_SSLCipherSuite(cmd_parms *, SSLDirConfigRec *, char *);
 const char  *ssl_cmd_SSLCertificateFile(cmd_parms *, char *, char *);
 const char  *ssl_cmd_SSLCertificateKeyFile(cmd_parms *, char *, char *);
-const char  *ssl_cmd_SSLCACertificatePath(cmd_parms *, char *, char *);
-const char  *ssl_cmd_SSLCACertificateFile(cmd_parms *, char *, char *);
-const char  *ssl_cmd_SSLVerifyClient(cmd_parms *, char *, char *level);
-const char  *ssl_cmd_SSLVerifyDepth(cmd_parms *, char *, char *);
+const char  *ssl_cmd_SSLCACertificatePath(cmd_parms *, SSLDirConfigRec *, char *);
+const char  *ssl_cmd_SSLCACertificateFile(cmd_parms *, SSLDirConfigRec *, char *);
+const char  *ssl_cmd_SSLVerifyClient(cmd_parms *, SSLDirConfigRec *, char *level);
+const char  *ssl_cmd_SSLVerifyDepth(cmd_parms *, SSLDirConfigRec *, char *);
 const char  *ssl_cmd_SSLSessionCache(cmd_parms *, char *, char *);
 const char  *ssl_cmd_SSLSessionCacheTimeout(cmd_parms *, char *, char *);
 const char  *ssl_cmd_SSLLog(cmd_parms *, char *, char *);
@@ -595,6 +609,9 @@ void         ssl_io_register(void);
 void         ssl_io_unregister(void);
 long         ssl_io_data_cb(BIO *, int, char *, int, long, long);
 
+/*  PRNG  */
+int          ssl_rand_seed(server_rec *, pool *, SSLRandSeedCtx);
+
 /*  Extensions  */
 void         ssl_ext_register(void);
 void         ssl_ext_unregister(void);
