@@ -115,6 +115,7 @@ void ssl_config_global_create(void)
         pPool = ap_make_sub_pool(NULL);
         mc = (SSLModConfigRec *)ap_palloc(pPool, sizeof(SSLModConfigRec));
         mc->pPool = pPool;
+        mc->bFixed = FALSE;
 
         /*
          * initialize per-module configuration
@@ -127,6 +128,7 @@ void ssl_config_global_create(void)
         mc->szMutexFile            = NULL;
         mc->nMutexFD               = -1;
         mc->nMutexSEMID            = -1;
+        mc->aRandSeed              = ap_make_array(pPool, 4, sizeof(SSLRandSeed));
 
         mc->tPrivateKey            = ssl_ds_table_make(pPool, sizeof(RSA *));
         mc->tPublicCert            = ssl_ds_table_make(pPool, sizeof(X509 *));
@@ -139,6 +141,19 @@ void ssl_config_global_create(void)
     return;
 }
 
+void ssl_config_global_fix(void)
+{
+    SSLModConfigRec *mc = myModConfig();
+    mc->bFixed = TRUE;
+    return;
+}
+
+BOOL ssl_config_global_isfixed(void)
+{
+    SSLModConfigRec *mc = myModConfig();
+    return (mc->bFixed);
+}
+
 
 /*  _________________________________________________________________
 **
@@ -222,6 +237,14 @@ void *ssl_config_perdir_create(pool *p, char *dir)
     dc->nOptionsAdd   = SSL_OPT_NONE;
     dc->nOptionsDel   = SSL_OPT_NONE;
 
+    dc->szCipherSuite          = NULL;
+    dc->nVerifyClient          = SSL_CVERIFY_UNSET;
+    dc->nVerifyDepth           = UNSET;
+#ifdef SSL_EXPERIMENTAL
+    dc->szCACertificatePath    = NULL;
+    dc->szCACertificateFile    = NULL;
+#endif
+
     return dc;
 }
 
@@ -249,6 +272,14 @@ void *ssl_config_perdir_merge(pool *p, void *basev, void *addv)
         new->nOptionsDel = add->nOptionsDel;
     }
 
+    cfgMergeString(szCipherSuite);
+    cfgMerge(nVerifyClient, SSL_CVERIFY_UNSET);
+    cfgMergeInt(nVerifyDepth);
+#ifdef SSL_EXPERIMENTAL
+    cfgMergeString(szCACertificatePath);
+    cfgMergeString(szCACertificateFile);
+#endif
+
     return new;
 }
 
@@ -277,12 +308,14 @@ const char *ssl_cmd_SSLMutex(
 
     if ((err = ap_check_cmd_context(cmd, GLOBAL_ONLY)) != NULL)
         return err;
+    if (ssl_config_global_isfixed())
+        return NULL;
     if (strcEQ(arg, "none")) {
         mc->nMutexMode  = SSL_MUTEXMODE_NONE;
     }
     else if (strlen(arg) > 5 && strcEQn(arg, "file:", 5)) {
         mc->nMutexMode  = SSL_MUTEXMODE_FILE;
-        mc->szMutexFile = ap_psprintf(cmd->pool, "%s.%lu",
+        mc->szMutexFile = ap_psprintf(mc->pPool, "%s.%lu",
                                       ap_server_root_relative(cmd->pool, arg+5), 
                                       (unsigned long)getpid());
     }
@@ -322,6 +355,58 @@ const char *ssl_cmd_SSLPassPhraseDialog(
     return NULL;
 }
 
+const char *ssl_cmd_SSLRandomSeed(
+    cmd_parms *cmd, char *struct_ptr, char *arg1, char *arg2, char *arg3)
+{
+    SSLModConfigRec *mc = myModConfig();
+    const char *err;
+    SSLRandSeed *pRS;
+
+    if ((err = ap_check_cmd_context(cmd, GLOBAL_ONLY)) != NULL)
+        return err;
+    if (ssl_config_global_isfixed())
+        return NULL;
+    pRS = ap_push_array(mc->aRandSeed);
+    if (strcEQ(arg1, "startup"))
+        pRS->nCtx = SSL_RSCTX_STARTUP;
+    else if (strcEQ(arg1, "connect"))
+        pRS->nCtx = SSL_RSCTX_CONNECT;
+    else
+        return ap_pstrcat(cmd->pool, "SSLRandomSeed: "
+                          "invalid context: `", arg1, "'");
+    if (strlen(arg2) > 5 && strEQn(arg2, "file:", 5)) {
+        pRS->nSrc   = SSL_RSSRC_FILE;
+        pRS->cpPath = ap_pstrdup(mc->pPool, ap_server_root_relative(cmd->pool, arg2+5));
+    }
+    else if (strlen(arg2) > 5 && strEQn(arg2, "exec:", 5)) {
+        pRS->nSrc   = SSL_RSSRC_EXEC;
+        pRS->cpPath = ap_pstrdup(mc->pPool, ap_server_root_relative(cmd->pool, arg2+5));
+    }
+    else if (strcEQ(arg2, "builtin")) {
+        pRS->nSrc   = SSL_RSSRC_BUILTIN;
+        pRS->cpPath = NULL;
+    }
+    else {
+        pRS->nSrc   = SSL_RSSRC_FILE;
+        pRS->cpPath = ap_pstrdup(mc->pPool, ap_server_root_relative(cmd->pool, arg2));
+    }
+    if (pRS->nSrc != SSL_RSSRC_BUILTIN)
+        if (!ssl_util_path_check(SSL_PCM_EXISTS, pRS->cpPath))
+            return ap_pstrcat(cmd->pool, "SSLRandomSeed: source path '",
+                              pRS->cpPath, "' not exists", NULL);
+    if (arg3 == NULL) 
+        pRS->nBytes = 0; /* read whole file */
+    else {
+        if (pRS->nSrc == SSL_RSSRC_BUILTIN)
+            return "SSLRandomSeed: byte specification not "
+                   "allowd for builtin seed source";
+        pRS->nBytes = atoi(arg3);
+        if (pRS->nBytes < 0)
+            return "SSLRandomSeed: invalid number of bytes specified";
+    }
+    return NULL;
+}
+
 const char *ssl_cmd_SSLEngine(
     cmd_parms *cmd, char *struct_ptr, int flag)
 {
@@ -332,10 +417,14 @@ const char *ssl_cmd_SSLEngine(
 }
 
 const char *ssl_cmd_SSLCipherSuite(
-    cmd_parms *cmd, char *struct_ptr, char *arg)
+    cmd_parms *cmd, SSLDirConfigRec *dc, char *arg)
 {
     SSLSrvConfigRec *sc = mySrvConfig(cmd->server);
-    sc->szCipherSuite = arg;
+
+    if (cmd->path == NULL || dc == NULL)
+        sc->szCipherSuite = arg;
+    else
+        dc->szCipherSuite = arg;
     return NULL;
 }
 
@@ -368,7 +457,7 @@ const char *ssl_cmd_SSLCertificateKeyFile(
 }
 
 const char *ssl_cmd_SSLCACertificatePath(
-    cmd_parms *cmd, char *struct_ptr, char *arg)
+    cmd_parms *cmd, SSLDirConfigRec *dc, char *arg)
 {
     SSLSrvConfigRec *sc = mySrvConfig(cmd->server);
     char *cpPath;
@@ -377,12 +466,19 @@ const char *ssl_cmd_SSLCACertificatePath(
     if (!ssl_util_path_check(SSL_PCM_EXISTS|SSL_PCM_ISDIR, cpPath))
         return ap_pstrcat(cmd->pool, "SSLCACertificatePath: directory '",
                           cpPath, "' not exists", NULL);
+#ifdef SSL_EXPERIMENTAL
+    if (cmd->path == NULL || dc == NULL)
+        sc->szCACertificatePath = cpPath;
+    else
+        dc->szCACertificatePath = cpPath;
+#else
     sc->szCACertificatePath = cpPath;
+#endif
     return NULL;
 }
 
 const char *ssl_cmd_SSLCACertificateFile(
-    cmd_parms *cmd, char *struct_ptr, char *arg)
+    cmd_parms *cmd, SSLDirConfigRec *dc, char *arg)
 {
     SSLSrvConfigRec *sc = mySrvConfig(cmd->server);
     char *cpPath;
@@ -391,36 +487,53 @@ const char *ssl_cmd_SSLCACertificateFile(
     if (!ssl_util_path_check(SSL_PCM_EXISTS|SSL_PCM_ISREG|SSL_PCM_ISNONZERO, cpPath))
         return ap_pstrcat(cmd->pool, "SSLCACertificateKeyFile: file '",
                           cpPath, "' not exists or empty", NULL);
+#ifdef SSL_EXPERIMENTAL
+    if (cmd->path == NULL || dc == NULL)
+        sc->szCACertificateFile = cpPath;
+    else
+        dc->szCACertificateFile = cpPath;
+#else
     sc->szCACertificateFile = cpPath;
+#endif
     return NULL;
 }
 
 const char *ssl_cmd_SSLVerifyClient(
-    cmd_parms *cmd, char *struct_ptr, char *level)
+    cmd_parms *cmd, SSLDirConfigRec *dc, char *level)
 {
     SSLSrvConfigRec *sc = mySrvConfig(cmd->server);
+    int id;
 
     if (strEQ(level, "0") || strcEQ(level, "none"))
-        sc->nVerifyClient = SSL_CVERIFY_NONE;
+        id = SSL_CVERIFY_NONE;
     else if (strEQ(level, "1") || strcEQ(level, "optional"))
-        sc->nVerifyClient = SSL_CVERIFY_OPTIONAL;
+        id = SSL_CVERIFY_OPTIONAL;
     else if (strEQ(level, "2") || strcEQ(level, "require"))
-        sc->nVerifyClient = SSL_CVERIFY_REQUIRE;
+        id = SSL_CVERIFY_REQUIRE;
     else if (strEQ(level, "3") || strcEQ(level, "optional_no_ca"))
-        sc->nVerifyClient = SSL_CVERIFY_OPTIONAL_NO_CA;
+        id = SSL_CVERIFY_OPTIONAL_NO_CA;
     else
         return "SSLVerifyClient: Invalid argument";
+    if (cmd->path == NULL || dc == NULL)
+        sc->nVerifyClient = id;
+    else
+        dc->nVerifyClient = id;
     return NULL;
 }
 
 const char *ssl_cmd_SSLVerifyDepth(
-    cmd_parms *cmd, char *struct_ptr, char *arg)
+    cmd_parms *cmd, SSLDirConfigRec *dc, char *arg)
 {
     SSLSrvConfigRec *sc = mySrvConfig(cmd->server);
+    int d;
 
-    sc->nVerifyDepth = atoi(arg);
-    if (sc->nVerifyDepth < 0)
+    d = atoi(arg);
+    if (d < 0)
         return "SSLVerifyDepth: Invalid argument";
+    if (cmd->path == NULL || dc == NULL)
+        sc->nVerifyDepth = d;
+    else
+        dc->nVerifyDepth = d;
     return NULL;
 }
 
@@ -432,13 +545,16 @@ const char *ssl_cmd_SSLSessionCache(
 
     if ((err = ap_check_cmd_context(cmd, GLOBAL_ONLY)) != NULL)
         return err;
+    if (ssl_config_global_isfixed())
+        return NULL;
     if (strcEQ(arg, "none")) {
         mc->nSessionCacheMode      = SSL_SCMODE_NONE;
         mc->szSessionCacheDataFile = NULL;
     }
     else if (strlen(arg) > 4 && strcEQn(arg, "dbm:", 4)) {
         mc->nSessionCacheMode      = SSL_SCMODE_DBM;
-        mc->szSessionCacheDataFile = ap_server_root_relative(cmd->pool, arg+4);
+        mc->szSessionCacheDataFile = ap_pstrdup(mc->pPool,
+                                     ap_server_root_relative(cmd->pool, arg+4));
     }
     else
         return "SSLSessionCache: Invalid argument";
