@@ -71,15 +71,27 @@
 */
 
 static void  ssl_ext_mlc_register(void);
+static void  ssl_ext_mlc_unregister(void);
+static void  ssl_ext_mr_register(void);
+static void  ssl_ext_mr_unregister(void);
 static void  ssl_ext_mp_register(void);
+static void  ssl_ext_mp_unregister(void);
 
 void ssl_ext_register(void)
 {
     ssl_ext_mlc_register();
+    ssl_ext_mr_register();
     ssl_ext_mp_register();
     return;
 }
 
+void ssl_ext_unregister(void)
+{
+    ssl_ext_mlc_unregister();
+    ssl_ext_mr_unregister();
+    ssl_ext_mp_unregister();
+    return;
+}
 
 /*  _________________________________________________________________
 **
@@ -87,9 +99,8 @@ void ssl_ext_register(void)
 **  _________________________________________________________________
 */
 
-static void *ssl_ext_mlc_register_function(char k);
-static char *ssl_ext_mlc_function_c(request_rec *r, char *a);
-static char *ssl_ext_mlc_function_x(request_rec *r, char *a);
+static char *ssl_ext_mlc_log_c(request_rec *r, char *a);
+static char *ssl_ext_mlc_log_x(request_rec *r, char *a);
 
 /*
  * register us for the mod_log_config function registering phase
@@ -97,35 +108,32 @@ static char *ssl_ext_mlc_function_x(request_rec *r, char *a);
  */
 static void ssl_ext_mlc_register(void)
 {
-    ap_hook_register("ap::log_config::register_function",
-                     ssl_ext_mlc_register_function, AP_HOOK_NOCTX);
-    ap_hook_register("ap::log_config::expand_x_variable",
-                     ssl_ext_mlc_function_x, AP_HOOK_NOCTX);
+    ap_hook_register("ap::mod_log_config::log_c",
+                     ssl_ext_mlc_log_c, AP_HOOK_NOCTX);
+    ap_hook_register("ap::mod_log_config::log_x",
+                     ssl_ext_mlc_log_x, AP_HOOK_NOCTX);
     return;
 }
 
-/*
- * register us for %{..}c in the
- * mod_log_config function register phase
- */
-static void *ssl_ext_mlc_register_function(char k)
+static void ssl_ext_mlc_unregister(void)
 {
-    if (k == 'c')
-        return ((void *)ssl_ext_mlc_function_c);
-    return NULL;
+    ap_hook_unregister("ap::mod_log_config::log_c",
+                       ssl_ext_mlc_log_c);
+    ap_hook_unregister("ap::mod_log_config::log_x", 
+                       ssl_ext_mlc_log_x);
+    return;
 }
 
 /*
  * implement the %{..}c log function
  * (we are the only function)
  */
-static char *ssl_ext_mlc_function_c(request_rec *r, char *a)
+static char *ssl_ext_mlc_log_c(request_rec *r, char *a)
 {
     char *result;
 
     if (ap_ctx_get(r->connection->client->ctx, "ssl") == NULL)
         return NULL;
-
     result = NULL;
     if (strcmp(a, "version") == 0)
         result = ssl_var_lookup(r->pool, r->server, r->connection, r, "SSL_PROTOCOL");
@@ -136,10 +144,9 @@ static char *ssl_ext_mlc_function_c(request_rec *r, char *a)
     else if (strcmp(a, "issuerdn") == 0 || strcmp(a, "cacert") == 0)
         result = ssl_var_lookup(r->pool, r->server, r->connection, r, "SSL_CLIENT_I_DN");
     else if (strcmp(a, "errcode") == 0)
-        result = ap_ctx_get(r->connection->client->ctx, "ssl::verify::errcode");
+        result = "-";
     else if (strcmp(a, "errstr") == 0)
-        result = ap_ctx_get(r->connection->client->ctx, "ssl::verify::errstr");
-
+        result = ap_ctx_get(r->connection->client->ctx, "ssl::verify::error");
     return result;
 }
 
@@ -147,7 +154,7 @@ static char *ssl_ext_mlc_function_c(request_rec *r, char *a)
  * extend the implementation of the %{..}x log function
  * (there can be more functions)
  */
-static char *ssl_ext_mlc_function_x(request_rec *r, char *a)
+static char *ssl_ext_mlc_log_x(request_rec *r, char *a)
 {
     char *result;
 
@@ -157,6 +164,40 @@ static char *ssl_ext_mlc_function_x(request_rec *r, char *a)
     return result;
 }
 
+/*  _________________________________________________________________
+**
+**  SSL Extension to mod_rewrite
+**  _________________________________________________________________
+*/
+
+static char *ssl_ext_mr_lookup_variable(request_rec *r, char *var);
+
+/*
+ * register us for the mod_rewrite lookup_variable() function
+ */
+static void ssl_ext_mr_register(void)
+{
+    ap_hook_register("ap::mod_rewrite::lookup_variable",
+                     ssl_ext_mr_lookup_variable, AP_HOOK_NOCTX);
+    return;
+}
+
+static void ssl_ext_mr_unregister(void)
+{
+    ap_hook_unregister("ap::mod_rewrite::lookup_variable", 
+                       ssl_ext_mr_lookup_variable);
+    return;
+}
+
+static char *ssl_ext_mr_lookup_variable(request_rec *r, char *var)
+{
+    char *val;
+
+    val = ssl_var_lookup(r->pool, r->server, r->connection, r, var);
+    if (val[0] == NUL)
+        val = NULL;
+    return val;
+}
 
 /*  _________________________________________________________________
 **
@@ -176,26 +217,39 @@ static int   ssl_ext_mp_write_host_header(request_rec *r, BUFF *fb, char *host,
  */
 static void ssl_ext_mp_register(void)
 {
-    ap_hook_register("ap::proxy::canon",
+    ap_hook_register("ap::mod_proxy::canon",
                      ssl_ext_mp_canon, AP_HOOK_NOCTX);
-    ap_hook_register("ap::proxy::handler",
+    ap_hook_register("ap::mod_proxy::handler",
                      ssl_ext_mp_handler, AP_HOOK_NOCTX);
-    ap_hook_register("ap::proxy::http::handler::set_destport",
+    ap_hook_register("ap::mod_proxy::http::handler::set_destport",
                      ssl_ext_mp_set_destport, AP_HOOK_NOCTX);
-    ap_hook_register("ap::proxy::http::handler::new_connection",
+    ap_hook_register("ap::mod_proxy::http::handler::new_connection",
                      ssl_ext_mp_new_connection, AP_HOOK_NOCTX);
-    ap_hook_register("ap::proxy::http::handler::write_host_header",
+    ap_hook_register("ap::mod_proxy::http::handler::write_host_header",
                      ssl_ext_mp_write_host_header, AP_HOOK_NOCTX);
     return;
 }
 
+static void ssl_ext_mp_unregister(void)
+{
+    ap_hook_unregister("ap::mod_proxy::canon", ssl_ext_mp_canon);
+    ap_hook_unregister("ap::mod_proxy::handler", ssl_ext_mp_handler);
+    ap_hook_unregister("ap::mod_proxy::http::handler::set_destport",
+                       ssl_ext_mp_set_destport);
+    ap_hook_unregister("ap::mod_proxy::http::handler::new_connection",
+                       ssl_ext_mp_new_connection);
+    ap_hook_unregister("ap::mod_proxy::http::handler::write_host_header",
+                       ssl_ext_mp_write_host_header);
+    return;
+}
+
 static int ssl_ext_mp_canon(request_rec *r, char *url)
 {
     int rc;
 
     if (strncasecmp(url, "https:", 6) == 0) {
         rc = OK;
-        ap_hook_call("ap::proxy::http::canon",
+        ap_hook_call("ap::mod_proxy::http::canon",
                      &rc, r, url+6, "https", DEFAULT_HTTPS_PORT);
         return rc;
     }
@@ -209,7 +263,7 @@ static int ssl_ext_mp_handler(
 
     if (strcasecmp(protocol, "https") == 0) {
         ap_ctx_set(r->ctx, "ssl_enabled", (void *)1);
-        ap_hook_call("ap::proxy::http::handler",
+        ap_hook_call("ap::mod_proxy::http::handler",
                      &rc, r, cr, url, proxyhost, proxyport);
         return rc;
     }
