@@ -123,11 +123,16 @@ void ssl_init_Module(server_rec *s, pool *p)
     SSLSrvConfigRec *sc;
     server_rec *s2;
     char *cp;
-
-    ssl_config_global_create();
+    int n;
 
     mc->nInitCount++;
 
+    /*
+     * Any init round fixes the global config
+     */
+    ssl_config_global_create(); /* just to avoid problems */
+    ssl_config_global_fix();
+
     /*
      *  try to fix the configuration and open the dedicated SSL
      *  logfile as early as possible
@@ -223,16 +228,18 @@ void ssl_init_Module(server_rec *s, pool *p)
     ssl_mutex_init(s, p);
     ssl_scache_init(s, p);
 
+    /*
+     * Seed the Pseudo Random Number Generator (PRNG)
+     */
+    n = ssl_rand_seed(s, p, SSL_RSCTX_STARTUP);
+    ssl_log(s, SSL_LOG_INFO, "Init: Seeding PRNG with %d bytes of entropy", n);
+
     /*
      *  pre-generate the temporary RSA key
      */
     if (mc->pRSATmpKey == NULL) {
         ssl_log(s, SSL_LOG_INFO, "Init: Generating temporary (512 bit) RSA private key");
-#if (SSL_LIBRARY_VERSION >= 0x0900)
         mc->pRSATmpKey = RSA_generate_key(512, RSA_F4, NULL, NULL);
-#else
-        mc->pRSATmpKey = RSA_generate_key(512, RSA_F4, NULL);
-#endif
         if (mc->pRSATmpKey == NULL) {
             ssl_log(s, SSL_LOG_ERROR, "Init: Failed to generate temporary (512 bit) RSA private key");
             ssl_die();
