@@ -133,6 +133,7 @@ void ssl_hook_NewConnection(conn_rec *conn)
     SSL *ssl;
     char *cp;
     int rc;
+    int n;
 
     /*
      * Get context
@@ -160,13 +161,20 @@ void ssl_hook_NewConnection(conn_rec *conn)
     ssl_log(srvr, SSL_LOG_INFO, "Connection to child %d established (server %s)",
             conn->child_num, ssl_util_vhostid(conn->pool, srvr));
 
+    /*
+     * Seed the Pseudo Random Number Generator (PRNG)
+     */
+    n = ssl_rand_seed(srvr, conn->pool, SSL_RSCTX_CONNECT);
+    ssl_log(srvr, SSL_LOG_TRACE, "Seeding PRNG with %d bytes of entropy", n);
+
     /*
      * Create a new SSL connection with the configured server SSL context and
      * attach this to the socket. Additionally we register this attachment
      * so we can detach later.
      */
     ssl = SSL_new(sc->pSSLCtx);
-    SSL_set_app_data(ssl, conn);
+    SSL_set_app_data(ssl, conn);  /* conn_rec    (available now)     */
+    SSL_set_app_data2(ssl, NULL); /* request_rec (available later)   */
     SSL_set_fd(ssl, fb->fd);
     ap_ctx_set(fb->ctx, "ssl", ssl);
     ap_register_cleanup(conn->pool, (void *)conn,
@@ -242,7 +250,6 @@ void ssl_hook_NewConnection(conn_rec *conn)
                 conn->aborted = 1;
                 return;
             }
-#if (SSL_LIBRARY_VERSION >= 0x0900)
             else if (ERR_GET_REASON(ERR_peek_error()) == SSL_R_HTTP_REQUEST) {
                 /*
                  * The case where SSLeay has recognized a HTTP request:
@@ -287,7 +294,6 @@ void ssl_hook_NewConnection(conn_rec *conn)
                 /* finally: let Apache go on with processing */
                 return;
             }
-#endif
             else if (ap_ctx_get(ap_global_ctx, "ssl::handshake::timeout") == (void *)TRUE) {
                 ssl_log(srvr, SSL_LOG_ERROR, 
                         "SSL handshake timed out (client %s, server %s)",
@@ -447,9 +453,22 @@ void ssl_hook_CloseConnection(void *_conn)
  */
 int ssl_hook_ReadReq(request_rec *r)
 {
+    SSL *ssl;
+
+    /*
+     * Get the SSL connection structure and perform the 
+     * delayed interlinking from SSL back to request_rec
+     */
+    ssl = ap_ctx_get(r->connection->client->ctx, "ssl");
+    if (ssl != NULL)
+        SSL_set_app_data2(ssl, r);
+
+    /*
+     * Force the mod_ssl content handler when URL indicates this
+     */
     if (strEQn(r->uri, "/mod_ssl:", 9))
         r->handler = "mod_ssl:content-handler";
-    if (ap_ctx_get(r->connection->client->ctx, "ssl") != NULL) {
+    if (ssl != NULL) {
         ap_ctx_set(r->ctx, "ap::http::method",  "https");
         ap_ctx_set(r->ctx, "ap::default::port", "443");
     }
@@ -544,19 +563,39 @@ int ssl_hook_Auth(request_rec *r)
  */
 int ssl_hook_Access(request_rec *r)
 {
-    SSLDirConfigRec *rec  = myDirConfig(r);
-    SSLSrvConfigRec *sc = mySrvConfig(r->server);
+    SSLDirConfigRec *dc;
+    SSLSrvConfigRec *sc;
+    SSL *ssl;
+    SSL_CTX *ctx;
     array_header *apRequirement;
     SSLRequirement *pRequirements;
     SSLRequirement *pRequirement;
     char *cp;
     int ok;
     int i;
+    BOOL renegotiate; 
+#ifdef SSL_EXPERIMENTAL
+    BOOL reconfigured_locations;
+    STACK *skCAList;
+    char *cpCAPath;
+    char *cpCAFile;
+#endif
+    STACK *skCipherOld;
+    STACK *skCipher;
+    int nVerifyOld;
+    int nVerify;
+    int n;
+
+    dc  = myDirConfig(r);
+    sc  = mySrvConfig(r->server);
+    ssl = ap_ctx_get(r->connection->client->ctx, "ssl");
+	if (ssl != NULL)
+        ctx = SSL_get_SSL_CTX(ssl);
 
     /*
      * Support for SSLRequireSSL directive
      */
-    if (rec->bSSLRequired && ap_ctx_get(r->connection->client->ctx, "ssl") == NULL) {
+    if (dc->bSSLRequired && ssl == NULL) {
         ap_log_reason("SSL connection required", r->filename, r);
         return FORBIDDEN;
     }
@@ -566,13 +605,218 @@ int ssl_hook_Access(request_rec *r)
      */
     if (!sc->bEnabled)
         return DECLINED;
-    if (ap_ctx_get(r->connection->client->ctx, "ssl") == NULL)
+    if (ssl == NULL)
         return DECLINED;
 
+    /*
+     * Support for per-directory SSL connection parameters.  
+     * 
+     * This is implemented by forcing a SSL renegotation with the reconfigured
+     * parameter suite. But Apache's internal API processing makes our life
+     * very hard, because when internal sub-requests occur we nevertheless
+     * should avoid multiple unnecessary SSL handshakes (they need network I/O
+     * and time to perform). But the optimization for filtering out the
+     * unnecessary handshakes isn't such obvious.  Especially because while
+     * Apache is in its sub-request processing the client could force
+     * additional handshakes, too. And these take place perhaps without our
+     * notice. So the only possibility is to ask SSLeay/OpenSSL whether the
+     * renegotiation has to be performed or not. It has to performed when the
+     * parameters which were previously known are not those we've now
+     * reconfigured.
+     */
+    renegotiate = FALSE;
+#ifdef SSL_EXPERIMENTAL
+    reconfigured_locations = FALSE;
+#endif
+
+    /* 
+     * override of SSLCipherSuite
+     */
+    if (dc->szCipherSuite != NULL) {
+        /* remember old cipher suite for comparison */
+        if ((skCipherOld = SSL_get_ciphers(ssl)) != NULL)
+            skCipherOld = sk_dup(skCipherOld);
+        /* configure new cipher suite */
+        if (!SSL_set_cipher_list(ssl, dc->szCipherSuite)) {
+            ssl_log(r->server, SSL_LOG_WARN|SSL_ADD_SSLERR,
+                    "Unable to reconfigure (per-directory) permitted SSL ciphers");
+            return FORBIDDEN;
+        }
+        /* determine whether the cipher suite was actually changed */
+        skCipher = SSL_get_ciphers(ssl);
+        if ((skCipherOld == NULL && skCipher != NULL) ||
+            (skCipherOld != NULL && skCipher == NULL)   )
+            renegotiate = TRUE;
+        else if (skCipherOld != NULL && skCipher != NULL) {
+            for (n = 0; n < sk_num(skCipher); n++) {
+                if (sk_find(skCipherOld, sk_value(skCipher, n)) < 0) {
+                    renegotiate = TRUE;
+                    break;
+                }
+            }
+            for (n = 0; n < sk_num(skCipherOld); n++) {
+                if (sk_find(skCipher, sk_value(skCipherOld, n)) < 0) {
+                    renegotiate = TRUE;
+                    break;
+                }
+            }
+        }
+        /* free old cipher suite */
+        if (skCipherOld != NULL)
+            sk_free(skCipherOld);
+    }
+
+    /* 
+     * override of SSLVerifyClient
+     */
+    if (dc->nVerifyClient != SSL_CVERIFY_UNSET) {
+        /* remember old verify mode */
+        nVerifyOld = SSL_get_verify_mode(ssl);
+        /* configure new verify mode */
+        nVerify = SSL_VERIFY_NONE;
+        if (dc->nVerifyClient == SSL_CVERIFY_REQUIRE)
+            nVerify |= SSL_VERIFY_PEER|SSL_VERIFY_FAIL_IF_NO_PEER_CERT;
+        if (   (dc->nVerifyClient == SSL_CVERIFY_OPTIONAL)
+            || (dc->nVerifyClient == SSL_CVERIFY_OPTIONAL_NO_CA) )
+            nVerify |= SSL_VERIFY_PEER;
+        SSL_set_verify(ssl, nVerify, ssl_callback_SSLVerify);
+        SSL_set_verify_result(ssl, X509_V_OK);
+        /* determine whether the verify mode was actually changed */
+        if (nVerify != nVerifyOld) 
+            renegotiate = TRUE;
+    }
+
+    /*
+     * override of SSLVerifyDepth:
+     * This is handled by us manually inside the verify callback
+     * function and not by SSLeay internally. And our function is
+     * aware of both the per-server and per-directory contexts.
+     * All we have to do is to force the renegotiation when the
+     * maximum allowed depth is changed.
+     */
+    if (dc->nVerifyDepth != UNSET) {
+        if (dc->nVerifyDepth != sc->nVerifyDepth)
+            renegotiate = TRUE;
+    }
+
+    /*
+     *  override SSLCACertificateFile & SSLCACertificatePath
+     *  This is tagged experimental because it has to use an ugly kludge: We
+     *  have to change the locations inside the SSL_CTX* (per-server global)
+     *  instead inside SSL* (per-connection local) and reconfigure it to the
+     *  old values later. That's problematic at least for the threaded process
+     *  model of Apache under Win32 or when an error occurs. But unless
+     *  OpenSSL provides a SSL_load_verify_locations() function we've no other
+     *  chance to provide this functionality...
+     */
+#ifdef SSL_EXPERIMENTAL
+    if (   (   dc->szCACertificateFile != NULL 
+            && (   sc->szCACertificateFile == NULL 
+                || (   sc->szCACertificateFile != NULL 
+                    && strNE(dc->szCACertificateFile, sc->szCACertificateFile))))
+        || (   dc->szCACertificatePath != NULL 
+            && (   sc->szCACertificatePath == NULL 
+                || (   sc->szCACertificatePath != NULL 
+                    && strNE(dc->szCACertificatePath, sc->szCACertificatePath)))) ) {
+        cpCAFile = dc->szCACertificateFile != NULL ?
+                   dc->szCACertificateFile : sc->szCACertificateFile;
+        cpCAPath = dc->szCACertificatePath != NULL ?
+                   dc->szCACertificatePath : sc->szCACertificatePath;
+        /*
+           FIXME: This should be...
+           if (!SSL_load_verify_locations(ssl, cpCAFile, cpCAPath)) { 
+           ...but SSLeay/OpenSSL still doesn't provide this!
+         */
+        if (!SSL_CTX_load_verify_locations(ctx, cpCAFile, cpCAPath)) {
+            ssl_log(r->server, SSL_LOG_ERROR|SSL_ADD_SSLERR,
+                    "Unable to reconfigure verify locations "
+                    "for client authentication");
+            return FORBIDDEN;
+        }
+        if ((skCAList = ssl_init_FindCAList(r->server, r->pool, 
+                                            cpCAFile, cpCAPath)) == NULL) {
+            ssl_log(r->server, SSL_LOG_ERROR,
+                    "Unable to determine list of available "
+                    "CA certificates for client authentication");
+            return FORBIDDEN;
+        }
+        SSL_set_client_CA_list(ssl, skCAList);
+        renegotiate = TRUE;
+        reconfigured_locations = TRUE;
+    }
+#endif /* SSL_EXPERIMENTAL */
+
+    /* 
+     * now do the renegotiation if anything was actually reconfigured
+     */
+    if (renegotiate) {
+        /* 
+         * Now we force the SSL renegotation by sending the Hello Request
+         * message to the client. Here we have to do a workaround: Actually
+         * SSLeay returns immediately after sending the Hello Request (the
+         * intent AFAIK is because the SSL/TLS protocol says it's not a must
+         * that the client replies to a Hello Request). But because we insist
+         * on a reply (anything else is an error for us) we have to go to the
+         * ACCEPT state manually. Using SSL_set_accept_state() doesn't work
+         * here because it resets too much of the connection.  So we set the
+         * state explicitly and continue the handshake manually.
+         */
+        ssl_log(r->server, SSL_LOG_INFO, "Requesting connection re-negotiation");
+        SSL_renegotiate(ssl);
+        SSL_do_handshake(ssl);
+        if (SSL_get_state(ssl) != SSL_ST_OK) {
+            ssl_log(r->server, SSL_LOG_ERROR, "Re-negotation request failed");
+            return FORBIDDEN;
+        }
+        ssl_log(r->server, SSL_LOG_INFO, "Awaiting re-negotiation handshake");
+        SSL_set_state(ssl, SSL_ST_ACCEPT);
+        SSL_do_handshake(ssl);
+        if (SSL_get_state(ssl) != SSL_ST_OK) {
+            ssl_log(r->server, SSL_LOG_ERROR, 
+                    "Re-negotiation handshake failed: Not accepted by client!?");
+            return FORBIDDEN;
+        }
+
+        /*
+         * Finally check for acceptable renegotiation results
+         */
+        if (dc->nVerifyClient != SSL_CVERIFY_NONE) {
+            if (SSL_get_verify_result(ssl) != X509_V_OK) {
+                ssl_log(r->server, SSL_LOG_ERROR, 
+                        "Re-negotiation handshake failed: Client verification failed");
+                return FORBIDDEN;
+            }
+            if (   dc->nVerifyClient == SSL_CVERIFY_REQUIRE 
+                && SSL_get_peer_certificate(ssl) == NULL   ) {
+                ssl_log(r->server, SSL_LOG_ERROR, 
+                        "Re-negotiation handshake failed: Client certificate missing");
+                return FORBIDDEN;
+            }
+        }
+    }
+
+    /*
+     * Under old SSLeay we had to change the X509_STORE inside the SSL_CTX
+     * instead inside the SSL structure, so we have to reconfigure it to the
+     * old values. This should be changed with forthcoming OpenSSL version
+     * when better functionality is avaiable.
+     */
+#ifdef SSL_EXPERIMENTAL
+    if (renegotiate && reconfigured_locations) {
+        if (!SSL_CTX_load_verify_locations(ctx,
+                sc->szCACertificateFile, sc->szCACertificatePath)) {
+            ssl_log(r->server, SSL_LOG_ERROR|SSL_ADD_SSLERR,
+                    "Unable to reconfigure verify locations "
+                    "to per-server configuration parameters");
+            return FORBIDDEN;
+        }
+    }
+#endif /* SSL_EXPERIMENTAL */
+
     /*
      * Check SSLRequire boolean expressions
      */
-    apRequirement = rec->aRequirement;
+    apRequirement = dc->aRequirement;
     pRequirements  = (SSLRequirement *)apRequirement->elts;
     for (i = 0; i < apRequirement->nelts; i++) {
         pRequirement = &pRequirements[i];
@@ -734,20 +978,25 @@ int ssl_callback_SSLVerify(int ok, X509_STORE_CTX *ctx)
     SSL *ssl;
     conn_rec *conn;
     server_rec *s;
+    request_rec *r;
     SSLSrvConfigRec *sc;
+    SSLDirConfigRec *dc;
     X509 *xs;
     int errnum;
     int errdepth;
     char *cp;
     char *cp2;
+    int depth;
 
     /*
      * Get Apache context back through SSLeay context
      */
     ssl  = (SSL *)X509_STORE_CTX_get_app_data(ctx);
     conn = (conn_rec *)SSL_get_app_data(ssl);
+    r    = (request_rec *)SSL_get_app_data2(ssl);
     s    = conn->server;
     sc   = mySrvConfig(s);
+    dc   = (r != NULL ? myDirConfig(r) : NULL);
 
     /*
      * Get verify ingredients
@@ -796,9 +1045,15 @@ int ssl_callback_SSLVerify(int ok, X509_STORE_CTX *ctx)
     /*
      * Finally check the depth of the certificate verification
      */
-    if (errdepth > sc->nVerifyDepth) {
+    if (dc != NULL && dc->nVerifyDepth != UNSET)
+        depth = dc->nVerifyDepth;
+    else 
+        depth = sc->nVerifyDepth;
+    if (errdepth > depth) {
         ssl_log(s, SSL_LOG_ERROR,
-                "Certificate Verification: Certificate Chain too long");
+                "Certificate Verification: Certificate Chain too long "
+                "(chain has %d certificates, but maximum allowed are only %d)", 
+                errdepth, depth);
         ap_ctx_set(conn->client->ctx, "ssl::verify::error",
                    X509_verify_cert_error_string(X509_V_ERR_CERT_CHAIN_TOO_LONG));
         ok = FALSE;
