@@ -156,7 +156,7 @@ void ssl_mutex_file_open(server_rec *s, pool *p)
     if ((mc->nMutexFD = ap_popenf(p, mc->szMutexFile,
                                            O_WRONLY, SSL_MUTEX_LOCK_MODE)) < 0) {
         ssl_log(s, SSL_LOG_ERROR|SSL_ADD_ERRNO,
-                "Child could not create SSLMutex lockfile %s",
+                "Child could not open SSLMutex lockfile %s",
                 mc->szMutexFile);
         ssl_die();
     }
