@@ -257,10 +257,12 @@ void ssl_scache_dbm_store(server_rec *s, SessionCacheInfo *SCI)
     dbmkey.dsize = SCI->nKey;
 
     /* create DBM value */
-    dbmval.dsize = sizeof(SCI->tExpiresAt)+SCI->nData;
+    dbmval.dsize = sizeof(time_t)+SCI->nData;
     dbmval.dptr  = (UCHAR *)malloc(dbmval.dsize);
-    memcpy(dbmval.dptr, &SCI->tExpiresAt, sizeof(SCI->tExpiresAt));
-    memcpy((char *)dbmval.dptr+sizeof(SCI->tExpiresAt), SCI->ucaData, SCI->nData);
+    if (dbmval.dptr == NULL)
+        return;
+    memcpy(dbmval.dptr, &SCI->tExpiresAt, sizeof(time_t));
+    memcpy((char *)dbmval.dptr+sizeof(time_t), SCI->ucaData, SCI->nData);
 
     /* and store it to the DBM file */
     ssl_mutex_on();
@@ -313,15 +315,18 @@ void ssl_scache_dbm_retrieve(server_rec *s, SessionCacheInfo *SCI)
     ssl_mutex_off();
 
     /* immediately return if not found */
-    if (dbmval.dptr == NULL)
+    if (dbmval.dptr == NULL || dbmval.dsize < sizeof(time_t))
         return;
 
     /* copy over the information to the SCI */
-    SCI->nData   = dbmval.dsize-sizeof(SCI->tExpiresAt);
-    SCI->ucaData = (UCHAR *)malloc(dbmval.dsize-sizeof(SCI->tExpiresAt));
-    memcpy(SCI->ucaData, (char *)dbmval.dptr+sizeof(SCI->tExpiresAt),
-           dbmval.dsize-sizeof(SCI->tExpiresAt));
-    memcpy(&SCI->tExpiresAt, dbmval.dptr, sizeof(SCI->tExpiresAt));
+    SCI->nData   = dbmval.dsize-sizeof(time_t);
+    SCI->ucaData = (UCHAR *)malloc(SCI->nData);
+    if (SCI->ucaData == NULL) {
+        SCI->nData = 0;
+        return;
+    }
+    memcpy(SCI->ucaData, (char *)dbmval.dptr+sizeof(time_t), SCI->nData);
+    memcpy(&SCI->tExpiresAt, dbmval.dptr, sizeof(time_t));
 
     return;
 }
@@ -385,7 +390,13 @@ void ssl_scache_dbm_expire(server_rec *s)
     dbmkey = ssl_dbm_firstkey(dbm);
     for ( ; dbmkey.dptr != NULL; dbmkey = ssl_dbm_nextkey(dbm)) {
         dbmval = ssl_dbm_fetch(dbm, dbmkey);
-        memcpy(&tExpiresAt, dbmval.dptr, sizeof(tExpiresAt));
+        if (dbmval.dptr == NULL)
+            continue;
+        if (dbmval.dsize < sizeof(time_t)) {
+            ssl_dbm_delete(dbm, dbmkey);
+            continue;
+        }
+        memcpy(&tExpiresAt, dbmval.dptr, sizeof(time_t));
         if (tExpiresAt >= tNow)
             ssl_dbm_delete(dbm, dbmkey);
     }
